local L                 = AceLibrary("AceLocale-2.2"):new("eeMon")
local RL                = AceLibrary("Roster-2.1")
local BC                = AceLibrary("Babble-Class-2.2")



--[[
desc:    Table to hold our default database values
]]--
local defaults = 
{
   Lock = false,
   
   ScanUnits = 5,
   ScanRange = 45,
   ScanHealth = 90,
   ScanFrequency = 0.3,
   
   SelfFilter = false,
   ClassFilter = {
      ["DRUID"] = false,
      ["HUNTER"] = false, 
      ["MAGE"] = false, 
      ["PALADIN"] = false, 
      ["PRIEST"] = false, 
      ["ROGUE"] = false, 
      ["SHAMAN"] = false, 
      ["WARLOCK"] = false, 
      ["WARRIOR"] = false,
      ["PET"] = true,
   },
   
   GroupFilter = {
      [1] = false,
      [2] = false,
      [3] = false,
      [4] = false,
      [5] = false,
      [6] = false,
      [7] = false,
      [8] = false,
   },
   
   DisplayScale = 1,
   DisplayWhen = L["Always"],
   DisplayClassColor = true,
   
   FrameAlpha = 1,
   FrameColor = {r = 0.0,g = 0.0,b = 0.0},
   FrameTitleOn = true,
   FrameBorderOn = true,
   
   BarAlpha = 1,
   BarStyle = L["Default"],
   BarTexture = L["Default"],
   BarWidth = 140,
   BarHeight = 14,
   BarSpacing = 5,
	
	NamePosition = L["OutsideBar"],
	HealthPosition = L["OutsideBar"],

}


local menu = 
{ 
   type='group',
   args = {
      
      lock = {
         type = 'toggle',
         name = L["Lock"],
         desc = L["LockDesc"],
         get  = function() return eeMon.db.profile.Lock end,
         set  = function(v) eeMon.db.profile.Lock = not eeMon.db.profile.Lock end,
         order= 1,
      },
      
      scan = {
         type = 'group',
         name = L["ScanOpt"],
         desc = L["ScanOptDesc"],
         order= 2,
         args = {
            
            units = {
               type = 'range',
               name = L["Units"],
               desc = L["UnitsDesc"],
               get  = function(v) return eeMon.db.profile.ScanUnits end,
               set  = function(v) eeMon.db.profile.ScanUnits = v eeMon:UpdateFrame() eeMon:UpdateBars() end,
               min  = 1.0,
               max  = 15.0,
               step = 1.0,
               isPercent = false,
               order= 3,
            },
            
            range = {
               type = 'range',
               name = L["Range"],
               desc = L["RangeDesc"],
               get  = function(v) return eeMon.db.profile.ScanRange end,
               set  = function(v) eeMon.db.profile.ScanRange = v end,
               min  = 5,
               max  = 60,
               step = 1,
               isPercent = false,
               order= 4,
            },
            
            health = {
               type = 'range',
               name = L["Health"],
               desc = L["HealthDesc"],
               get  = function(v) return eeMon.db.profile.ScanHealth end,
               set  = function(v) eeMon.db.profile.ScanHealth = v end,
               min  = 5.0,
               max  = 99.0,
               step = 1.0,
               isPercent = false,
               order= 5,
            },
            
            frequency = {
               type = 'range',
               name = L["Frequency"],
               desc = L["FrequencyDesc"],
               get  = function(v) return eeMon.db.profile.ScanFrequency end,
               set  = function(v) eeMon.db.profile.ScanFrequency = v eeMon:UpdateFrequency() end,
               min  = 0.2,
               max  = 2.0,
               step = 0.1,
               isPercent = false,
               order= 6,
            },
            
         }
      },
      
      filter = {
         type = 'group',
         name = L["FilterOpt"],
         desc = L["FilterOptDesc"],
         order= 7,
         args = {
            
            class = {
               type='group',
               name = L["ClassOpt"],
               desc = L["ClassOptDesc"],
               order = 8,
               args = 
               {
                  
                  druid =
                  {
                     type = 'toggle',
                     name = BC["Druid"],
                     desc = L["FilterDruid"],
                     get  = function(u) return eeMon.db.profile.ClassFilter["DRUID"] end,
                     set  = function(u) eeMon.db.profile.ClassFilter["DRUID"] = not eeMon.db.profile.ClassFilter["DRUID"] end,
                     order= 9,
                  },
                  
                  hunter = 
                  {
                     type = 'toggle',
                     name = BC["Hunter"],
                     desc = L["FilterHunter"],
                     get  = function(u) return eeMon.db.profile.ClassFilter["HUNTER"] end,
                     set  = function(u) eeMon.db.profile.ClassFilter["HUNTER"] = not eeMon.db.profile.ClassFilter["HUNTER"] end,
                     order= 10,
                  },
                  
                  mage =
                  {
                     type = 'toggle',
                     name = BC["Mage"],
                     desc = L["FilterMage"],
                     get  = function(u) return eeMon.db.profile.ClassFilter["MAGE"] end,
                     set  = function(u) eeMon.db.profile.ClassFilter["MAGE"] = not eeMon.db.profile.ClassFilter["MAGE"] end,
                     order= 29,
                  },
                  
                  paladin = 
                  {
                     type = 'toggle',
                     name = BC["Paladin"],
                     desc = L["FilterPaladin"],
                     get  = function(u) return eeMon.db.profile.ClassFilter["PALADIN"] end,
                     set  = function(u) eeMon.db.profile.ClassFilter["PALADIN"] = not eeMon.db.profile.ClassFilter["PALADIN"] end,
                     order= 11,
                  },
                  
                  priest = 
                  {
                     type = 'toggle',
                     name = BC["Priest"],
                     desc = L["FilterPriest"],
                     get  = function(u) return eeMon.db.profile.ClassFilter["PRIEST"] end,
                     set  = function(u) eeMon.db.profile.ClassFilter["PRIEST"] = not eeMon.db.profile.ClassFilter["PRIEST"] end,
                     order= 12,
                  },
                  
                  rogue = 
                  {
                     type = 'toggle',
                     name = BC["Rogue"],
                     desc = L["FilterRogue"],
                     get  = function(u) return eeMon.db.profile.ClassFilter["ROGUE"] end,
                     set  = function(u) eeMon.db.profile.ClassFilter["ROGUE"] = not eeMon.db.profile.ClassFilter["ROGUE"] end,
                     order= 13,
                  },
                  
                  shaman =
                  {
                     type = 'toggle',
                     name = BC["Shaman"],
                     desc = L["FilterShaman"],
                     get  = function(u) return eeMon.db.profile.ClassFilter["SHAMAN"] end,
                     set  = function(u) eeMon.db.profile.ClassFilter["SHAMAN"] = not eeMon.db.profile.ClassFilter["SHAMAN"] end,
                     order= 33,
                  },
                  
                  warlock = 
                  {
                     type = 'toggle',
                     name = BC["Warlock"],
                     desc = L["FilterWarlock"],
                     get  = function(u) return eeMon.db.profile.ClassFilter["WARLOCK"] end,
                     set  = function(u) eeMon.db.profile.ClassFilter["WARLOCK"] = not eeMon.db.profile.ClassFilter["WARLOCK"] end,
                     order= 14,
                  },
                  
                  warrior =
                  {
                     type = 'toggle',
                     name = BC["Warrior"],
                     desc = L["FilterWarrior"],
                     get  = function(u) return eeMon.db.profile.ClassFilter["WARRIOR"] end,
                     set  = function(u) eeMon.db.profile.ClassFilter["WARRIOR"] = not eeMon.db.profile.ClassFilter["WARRIOR"] end,
                     order= 15,
                  },
                  
               }
            },
            
            group = {
               type='group',
               name = L["GroupOpt"],
               desc = L["GroupOptDesc"],
               order = 16,
               args = 
               {
                  group1 =
                  {
                     type = 'toggle',
                     name = L["Group1"],
                     desc = L["Group1Desc"],
                     get  = function(u) return eeMon.db.profile.GroupFilter[1] end,
                     set  = function(u) eeMon.db.profile.GroupFilter[1] = not eeMon.db.profile.GroupFilter[1] end,
                     order= 17,
                  },
                  
                  group2 =
                  {
                     type = 'toggle',
                     name = L["Group2"],
                     desc = L["Group2Desc"],
                     get  = function(u) return eeMon.db.profile.GroupFilter[2] end,
                     set  = function(u) eeMon.db.profile.GroupFilter[2] = not eeMon.db.profile.GroupFilter[2] end,
                     order= 18,
                  },
                  
                  group3 =
                  {
                     type = 'toggle',
                     name = L["Group3"],
                     desc = L["Group3Desc"],
                     get  = function(u) return eeMon.db.profile.GroupFilter[3] end,
                     set  = function(u) eeMon.db.profile.GroupFilter[3] = not eeMon.db.profile.GroupFilter[3] end,
                     order= 19,
                  },
                  
                  group4 =
                  {
                     type = 'toggle',
                     name = L["Group4"],
                     desc = L["Group4Desc"],
                     get  = function(u) return eeMon.db.profile.GroupFilter[4] end,
                     set  = function(u) eeMon.db.profile.GroupFilter[4] = not eeMon.db.profile.GroupFilter[4] end,
                     order= 19,
                  },
                  
                  group5 =
                  {
                     type = 'toggle',
                     name = L["Group5"],
                     desc = L["Group5Desc"],
                     get  = function(u) return eeMon.db.profile.GroupFilter[5] end,
                     set  = function(u) eeMon.db.profile.GroupFilter[5] = not eeMon.db.profile.GroupFilter[5] end,
                     order= 20,
                  },
                  
                  group6 =
                  {
                     type = 'toggle',
                     name = L["Group6"],
                     desc = L["Group6Desc"],
                     get  = function(u) return eeMon.db.profile.GroupFilter[6] end,
                     set  = function(u) eeMon.db.profile.GroupFilter[6] = not eeMon.db.profile.GroupFilter[6] end,
                     order= 21,
                  },
                  
                  group7 =
                  {
                     type = 'toggle',
                     name = L["Group7"],
                     desc = L["Group7Desc"],
                     get  = function(u) return eeMon.db.profile.GroupFilter[7] end,
                     set  = function(u) eeMon.db.profile.GroupFilter[7] = not eeMon.db.profile.GroupFilter[7] end,
                     order= 22,
                  },
                  
                  group8 =
                  {
                     type = 'toggle',
                     name = L["Group8"],
                     desc = L["Group8Desc"],
                     get  = function(u) return eeMon.db.profile.GroupFilter[8] end,
                     set  = function(u) eeMon.db.profile.GroupFilter[8] = not eeMon.db.profile.GroupFilter[8] end,
                     order= 23,
                  },
               }
            },
            
            pets = {
               type = 'toggle',
               name = L["FilterPet"],
               desc = L["FilterPetDesc"],
               get  = function(u) return eeMon.db.profile.ClassFilter["PET"] end,
               set  = function(u) eeMon.db.profile.ClassFilter["PET"] = not eeMon.db.profile.ClassFilter["PET"] end,
               order= 24,
            },
            
            self = {
               type = 'toggle',
               name = L["FilterSelf"],
               desc = L["FilterSelfDesc"],
               get  = function(u) return eeMon.db.profile.SelfFilter end,
               set  = function(u) eeMon.db.profile.SelfFilter = not eeMon.db.profile.SelfFilter end,
               order= 25,
            },
            
         }
      },
      
      display = {
         type = 'group',
         name = L["DisplayOpt"],
         desc = L["DisplayOptDesc"],
         order= 26,
         args = {
            
            when = {
               type = 'text',
               name = L["DisplayWhen"],
               desc = L["DisplayWhenDesc"],
               get  = function(v) return eeMon.db.profile.DisplayWhen end,
               set  = function(v) eeMon.db.profile.DisplayWhen = v eeMon:CheckVisibility() end,
               validate = {L["Always"], L["Party"], L["Raid"], L["Never"]},
               order = 27,
            },
            
            scale = {
               type = 'range',
               name = L["DisplayScale"],
               desc = L["DisplayScaleDesc"],
               get  = function(v) return eeMon.db.profile.DisplayScale end,
               set  = function(v) eeMon.db.profile.DisplayScale = v eeMon:Scale() end,
               min  = 0.1,
               max  = 2.0,
               step = 0.01,
               isPercent = true,
               order= 28,
            },
            
            frame = {
               type='group',
               name = L["FrameOpt"],
               desc = L["FrameOptDesc"],
               order = 29,
               args = {
                  
                  alpha = {
                     type = 'range',
                     name = L["FrameAlpha"],
                     desc = L["FrameAlphaDesc"],
                     get  = function(v) return eeMon.db.profile.FrameAlpha end,
                     set  = function(v) eeMon.db.profile.FrameAlpha = v eeMon:UpdateFrame() end,
                     min  = 0.0,
                     max  = 1.0,
                     step = 0.01,
                     isPercent = true,
                     order= 30,
                  },
                  
                  color = {
                     type = 'color',
                     name = L["FrameColor"],
                     desc = L["FrameColorDesc"],
                     get  = function(v) return eeMon.db.profile.FrameColor.r,eeMon.db.profile.FrameColor.g,eeMon.db.profile.FrameColor.b end,
                     set  = function(nr,ng,nb) eeMon.db.profile.FrameColor = {r=nr,g=ng,b=nb} eeMon:UpdateFrame() end,
                     order= 31,
                  },
                  
                  border = {
                     type = 'toggle',
                     name = L["FrameBorder"],
                     desc = L["FrameBorderDesc"],
                     get  = function(v) return eeMon.db.profile.FrameBorderOn end,
                     set  = function(v) eeMon.db.profile.FrameBorderOn = not eeMon.db.profile.FrameBorderOn eeMon:UpdateFrame() end,
                     order= 32,
                  },
                  
                  title = {
                     type = 'toggle',
                     name = L["FrameTitle"],
                     desc = L["FrameTitleDesc"],
                     get  = function(v) return eeMon.db.profile.FrameTitleOn end,
                     set  = function(v) eeMon.db.profile.FrameTitleOn = not eeMon.db.profile.FrameTitleOn eeMon:UpdateFrame() end,
                     order= 33,
                  },
                  
               }
            },
            
            bars = {
               type='group',
               name = L["BarsOpt"],
               desc = L["BarsOptDesc"],
               order = 34,
               args = {
                  
                  alpha = {
                     type = 'range',
                     name = L["BarAlpha"],
                     desc = L["BarAlphaDesc"],
                     get  = function(v) return eeMon.db.profile.BarAlpha end,
                     set  = function(v) eeMon.db.profile.BarAlpha = v end,
                     min  = 0.2,
                     max  = 1.0,
                     step = 0.01,
                     isPercent = true,
                     order= 35,
                  },
                  
                  style = {
                     type = 'text',
                     name = L["BarStyle"],
                     desc = L["BarStyleDesc"],
                     get  = function(v) return eeMon.db.profile.BarStyle end,
                     set  = function(v) eeMon.db.profile.BarStyle = v eeMon:UpdateBars() end,
                     validate = {L["Default"],L["CTRaid"],L["Class"]},
                     order = 36,
                  },
                  
						
						texture = {
                     type = 'text',
                     name = L["BarTexture"],
                     desc = L["BarTextureDesc"],
                     get  = function(v) return eeMon.db.profile.BarTexture end,
                     set  = function(v) eeMon:SetBarTexture(v) eeMon:UpdateBars() end,
                     validate = { L["Default"],L["Charcoal"],L["Bars"],L["Bumps"],L["Button"], 
								L["BantoBar"],L["Cloud"],L["Cracked"],L["Dabs"],L["Diagonal"],
								L["Grid"],L["Hatched"],L["Perl"], L["Otravi"],L["Rain"],L["Skewed"],
								L["Smooth"],L["Smudge"],L["Water"],L["Wisps"] },
                     order = 37,
                  },
                  
                  width = {
                     type = 'range',
                     name = L["BarWidth"],
                     desc = L["BarWidthDesc"],
                     get  = function(v) return eeMon.db.profile.BarWidth end,
                     set  = function(v) eeMon.db.profile.BarWidth = v eeMon:UpdateFrame() eeMon:UpdateBars() end,
                     min  = 50,
                     max  = 200,
                     step = 5,
                     isPercent = false,
                     order= 38,
                  },
                  
                  height = {
                     type = 'range',
                     name = L["BarHeight"],
                     desc = L["BarHeightDesc"],
                     get  = function(v) return eeMon.db.profile.BarHeight end,
                     set  = function(v) eeMon.db.profile.BarHeight = v eeMon:UpdateFrame() eeMon:UpdateBars() end,
                     min  = 10,
                     max  = 30,
                     step = 1,
                     isPercent = false,
                     order= 39,
                  },
                  
                  spacing = {
                     type = 'range',
                     name = L["BarSpacing"],
                     desc = L["BarSpacingDesc"],
                     get  = function(v) return eeMon.db.profile.BarSpacing end,
                     set  = function(v) eeMon.db.profile.BarSpacing = v eeMon:UpdateFrame() eeMon:UpdateBars() end,
                     min  = 0,
                     max  = 12,
                     step = 1,
                     isPercent = false,
                     order= 40,
                  },
               }
            },
				
				
				
				namePosition = {
					type = 'text',
					name = L["NamePosition"],
					desc = L["NamePositionDesc"],
					get  = function(v) return eeMon.db.profile.NamePosition end,
					set  = function(v) eeMon.db.profile.NamePosition = v eeMon:UpdateFrame() eeMon:UpdateBars() end,
					validate = {L["OutsideBar"],L["InsideBar"]},
					order= 41,
				},
				
				missingHealthPosition = {
               type = 'text',
               name = L["HealthPosition"],
               desc = L["HealthPositionDesc"],
               get  = function(v) return eeMon.db.profile.HealthPosition end,
               set  = function(v) eeMon.db.profile.HealthPosition = v eeMon:UpdateFrame() eeMon:UpdateBars() end,
					validate = {L["OutsideBar"],L["InsideBar"],L["Hidden"]},
               order= 42,
            },
				
				classColorNames = {
               type = 'toggle',
               name = L["DisplayClassColor"],
               desc = L["DisplayClassColorDesc"],
               get  = function(v) return eeMon.db.profile.DisplayClassColor end,
               set  = function(v) eeMon.db.profile.DisplayClassColor = not eeMon.db.profile.DisplayClassColor end,
               order= 43,
            },
            
         }
      },
		
		test = {
			type = 'execute',
			name = L["Test"],
			desc = L["TestDesc"],
			func = function(v) eeMon:Test() end,
			order= 44,
		},
      
   }
   
}


--[[
desc:    ace2 constructor for our addon
]]--
eeMon = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0", "AceHook-2.1", "AceDebug-2.0", "FuBarPlugin-2.0")

--[[
desc:    ace2 options for our addons menu
]]--
eeMon.OnMenuRequest = menu
eeMon.name = L["eeMon"]
eeMon.hasIcon = true
eeMon.defaultMinimapPosition = 45
eeMon.cannotDetachTooltip = true
eeMon.independentProfile = true
eeMon.defaultPosition = "LEFT"
eeMon.hideWithoutStandby = true
eeMon.hasNoColor = true



--[[
name:    OnIniialize()
desc:    Called when our Ace2 plugin is initialized (at WoW startup)
--]]
function eeMon:OnInitialize()
	self:RegisterDB("eeMonDB")
	self:RegisterDefaults('profile', defaults )
	self:RegisterChatCommand( {L["/eeMon"]}, menu )
	if not FuBar then
		self.OnMenuRequest.args.hide.guiName = "Hide minimap icon"
		self.OnMenuRequest.args.hide.desc = "Hide minimap icon"
	end
end



--[[
name:    OnEnable()
desc:    Called when our Ace2 plugin is enabled
--]]
function eeMon:OnEnable()
   -- Register a change in party/raid roster from our library
   self:RegisterEvent("RosterLib_RosterUpdated")
   -- Build our gui
   self:BuildDisplay()
   -- Check frame visibility, show if necessary
   self:CheckVisibility()
end



--[[
name:    OnDisable()
desc:    Called when our Ace2 plugin is disabled
--]]
function eeMon:OnDisable()
   self:HideGUI()
end



--[[
name:		SetBarTexture()
desc:		Technique taken from oRA2.  We do a reverse-lookup on the localized 
			text, and use that variable name (which we make sure is the same name 
			as the image file) to actually set the image.
]]--
function eeMon:SetBarTexture(locTextureName)
	local texture
	-- Try to get the variable name for this localized text, which we can use to
	-- do a loopup in our texture table
	if L:HasReverseTranslation(locTextureName) then
		texture = L:GetReverseTranslation(locTextureName)
	-- If we couldn't do that (for whatever reason), we'll just set it to Default
	-- which we know exists.  Should never happen, and would mess up localized text
	-- for non-english.
	else
		texture = "Default"
	end
	eeMon.db.profile.BarTexture = texture
	
end

